# Concept: friend

The `friend` keyword in C++ is used to grant special access privileges to functions or classes. It allows non-member functions or other classes to access private and protected members of a class, which they normally wouldn't be able to access. While it can be useful in certain scenarios, it should be used judiciously as it can potentially break encapsulation.

## Key Characteristics

- Allows access to private and protected members of a class
- Can be applied to functions, member functions, or entire classes
- Not symmetric (if A is a friend of B, B is not automatically a friend of A)
- Not transitive (if A is a friend of B, and B is a friend of C, A is not automatically a friend of C)
- Not inherited (friendship is not passed down to derived classes)


## Example 1: Friend Function

```cpp
#include <iostream>

class Box {
private:
    double width;
    double height;
    double depth;

public:
    Box(double w, double h, double d) : width(w), height(h), depth(d) {}

    // Declaration of friend function
    friend double calculateVolume(const Box& box);
};

// Definition of friend function
double calculateVolume(const Box& box) {
    return box.width * box.height * box.depth;
}

int main() {
    Box myBox(3.0, 4.0, 5.0);
    std::cout << "Volume of myBox: " << calculateVolume(myBox) << std::endl;
    return 0;
}
```

### Explanation:

- `calculateVolume` is declared as a friend function inside the `Box` class.
- It can access private members of `Box` (`width`, `height`, `depth`).
- The function is defined outside the class scope.

## Example 2: Friend Class

```cpp
#include <iostream>

class Square; // Forward declaration

class Rectangle {
private:
    double width;
    double height;

public:
    Rectangle(double w, double h) : width(w), height(h) {}

    // Declaring Square as a friend class
    friend class Square;

    double getArea() const {
        return width * height;
    }
};

class Square {
private:
    double side;

public:
    Square(double s) : side(s) {}

    // This method can access private members of Rectangle
    bool isLargerThan(const Rectangle& rect) const {
        double squareArea = side * side;
        return squareArea > (rect.width * rect.height);
    }
};

int main() {
    Rectangle rect(4.0, 5.0);
    Square square(4.5);

    std::cout << "Rectangle area: " << rect.getArea() << std::endl;
    std::cout << "Is square larger? " << (square.isLargerThan(rect) ? "Yes" : "No") << std::endl;

    return 0;
}
```

### Explanation:

- `Square` is declared as a friend class of `Rectangle`.
- `Square` can access private members of `Rectangle`.
- The `isLargerThan` method in `Square` directly accesses `Rectangle`'s private members.

## Example 3: Friend Member Function

```cpp
#include <iostream>

class Box; // Forward declaration

class BoxFactory {
public:
    static Box createCube(double side);
    void printBoxVolume(const Box& box) const;
};

class Box {
private:
    double width;
    double height;
    double depth;

public:
    Box(double w, double h, double d) : width(w), height(h), depth(d) {}

    // Declaring a specific member function of BoxFactory as friend
    friend Box BoxFactory::createCube(double side);
    friend void BoxFactory::printBoxVolume(const Box& box) const;
};

Box BoxFactory::createCube(double side) {
    return Box(side, side, side);
}

void BoxFactory::printBoxVolume(const Box& box) const {
    double volume = box.width * box.height * box.depth;
    std::cout << "Box volume: " << volume << std::endl;
}

int main() {
    BoxFactory factory;
    Box cube = BoxFactory::createCube(3.0);
    factory.printBoxVolume(cube);

    return 0;
}
```

Explanation:
- Specific member functions of `BoxFactory` are declared as friends of `Box`.
- `createCube` and `printBoxVolume` can access private members of `Box`.
- This allows for more granular control over which functions have access.

## Additional Considerations

1. **Overuse of Friends**: Excessive use of friend functions or classes can break encapsulation and make the code harder to maintain.

2. **Design Implications**: Using friends might indicate that your class design could be improved. Consider if the functionality can be achieved through public methods instead.

3. **Compilation Dependencies**: Friend declarations can introduce compilation dependencies between classes.

4. **Const Correctness**: When declaring friend functions, ensure they are const-correct if they don't modify the object.

5. **Template Friends**: C++ allows for template friend functions and classes, which can be useful in generic programming.

## Summary

The `friend` keyword in C++ provides a mechanism to grant special access privileges to functions or classes:

1. Friend functions can access private and protected members of a class.
2. Entire classes can be declared as friends, giving all their members access.
3. Specific member functions of a class can be declared as friends.

### Key points to remember:

- Friendship is not symmetric, transitive, or inherited.
- It should be used sparingly to maintain encapsulation.
- Friend declarations don't affect the public interface of a class.
- It can be useful for operator overloading and certain design patterns.

### Best practices:

- Use friends judiciously and only when necessary.
- Consider alternative designs before resorting to friends.
- Document the use of friends clearly in your code.
- Be aware of the potential impact on code maintainability and encapsulation.

While the `friend` keyword can be a powerful tool in C++, it should be used thoughtfully. It's important to balance the need for access with the principles of encapsulation and data hiding in object-oriented design.

## Related

- What are the benefits of having friends in life
- How can I make new friends as an adult
- What qualities make a good friend
- How do friendships impact mental health
- What are some ways to strengthen existing friendships
